@@ -29,6 +29,7 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #include "log.h"
 #include "stream.h"
 #include "jhash.h"
+#include "filter.h"
 
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_aspath.h"
@@ -98,6 +99,12 @@ assegment_data_new (int num)
   return (XMALLOC (MTYPE_AS_SEG_DATA, ASSEGMENT_DATA_SIZE (num, 1)));
 }
 
+static void
+assegment_data_free (as_t *asdata)
+{
+  XFREE (MTYPE_AS_SEG_DATA, asdata);
+}
+
 /* Get a new segment. Note that 0 is an allowed length,
  * and will result in a segment with no allocated data segment.
  * the caller should immediately assign data to the segment, as the segment
@@ -126,7 +133,7 @@ assegment_free (struct assegment *seg)
     return;
   
   if (seg->as)
-    XFREE (MTYPE_AS_SEG_DATA, seg->as);
+    assegment_data_free (seg->as);
   memset (seg, 0xfe, sizeof(struct assegment));
   XFREE (MTYPE_AS_SEG, seg);
   
@@ -194,13 +201,14 @@ assegment_prepend_asns (struct assegment *seg, as_t asnum, int num)
   if (num >= AS_SEGMENT_MAX)
     return seg; /* we don't do huge prepends */
   
-  newas = assegment_data_new (seg->length + num);
-
+  if ((newas = assegment_data_new (seg->length + num)) == NULL)
+    return seg;
+  
   for (i = 0; i < num; i++)
     newas[i] = asnum;
 
   memcpy (newas + num, seg->as, ASSEGMENT_DATA_SIZE (seg->length, 1));
-  XFREE (MTYPE_AS_SEG_DATA, seg->as);
+  assegment_data_free (seg->as);
   seg->as = newas;
   seg->length += num;
 
@@ -419,7 +427,7 @@ aspath_count_confeds (struct aspath *aspath)
 }
 
 unsigned int
-aspath_count_hops (struct aspath *aspath)
+aspath_count_hops (const struct aspath *aspath)
 {
   int count = 0;
   struct assegment *seg = aspath->segments;
@@ -467,9 +475,8 @@ aspath_highest (struct aspath *aspath)
   while (seg)
     {
       for (i = 0; i < seg->length; i++)
-        if (seg->as[i] > highest
-            && (seg->as[i] < BGP_PRIVATE_AS_MIN
-                || seg->as[i] > BGP_PRIVATE_AS_MAX))
+        if (seg->as[i] > highest 
+            && !BGP_AS_IS_PRIVATE(seg->as[i]))
 	  highest = seg->as[i];
       seg = seg->next;
     }
@@ -896,7 +903,7 @@ aspath_put (struct stream *s, struct aspath *as, int use32bit )
               assegment_header_put (s, seg->type, AS_SEGMENT_MAX);
               assegment_data_put (s, seg->as, AS_SEGMENT_MAX, use32bit);
               written += AS_SEGMENT_MAX;
-              bytes += ASSEGMENT_SIZE (written, use32bit);
+              bytes += ASSEGMENT_SIZE (AS_SEGMENT_MAX, use32bit);
             }
           
           /* write the final segment, probably is also the first */
@@ -1057,6 +1064,9 @@ aspath_aggregate (struct aspath *as1, struct aspath *as2)
       if (match != minlen || match != seg1->length 
 	  || seg1->length != seg2->length)
 	break;
+      /* We are moving on to the next segment to reset match */
+      else
+        match = 0;
       
       seg1 = seg1->next;
       seg2 = seg2->next;
@@ -1091,6 +1101,128 @@ aspath_aggregate (struct aspath *as1, struct aspath *as2)
   return aspath;
 }
 
+/* Modify as1 using as2 for aggregation for multipath, keeping the
+ * AS-Path length the same, and so minimising change to the preference
+ * of the mpath aggregrate route.
+ */
+struct aspath *
+aspath_aggregate_mpath (struct aspath *as1, struct aspath *as2)
+{
+  int i;
+  int minlen;
+  int match;
+  int from1,from2;
+  struct assegment *seg1 = as1->segments;
+  struct assegment *seg2 = as2->segments;
+  struct aspath *aspath = NULL;
+  struct assegment *asset;
+  struct assegment *prevseg = NULL;
+
+  match = 0;
+  minlen = 0;
+  aspath = NULL;
+  asset = NULL;
+
+  /* First of all check common leading sequence. */
+  while (seg1 && seg2)
+    {
+      /* Check segment type. */
+      if (seg1->type != seg2->type)
+	break;
+
+      /* Minimum segment length. */
+      minlen = min (seg1->length, seg2->length);
+
+      for (match = 0; match < minlen; match++)
+	if (seg1->as[match] != seg2->as[match])
+	  break;
+
+      if (match)
+	{
+	  struct assegment *seg = assegment_new (seg1->type, 0);
+
+	  seg = assegment_append_asns (seg, seg1->as, match);
+
+	  if (! aspath)
+	    {
+	      aspath = aspath_new ();
+	      aspath->segments = seg;
+	     }
+	  else
+	    prevseg->next = seg;
+
+	  prevseg = seg;
+	}
+
+      if (match != minlen || match != seg1->length
+	  || seg1->length != seg2->length)
+	break;
+
+      seg1 = seg1->next;
+      seg2 = seg2->next;
+    }
+
+  if (! aspath)
+    aspath = aspath_new();
+
+  /* Make as-set using rest of all information. */
+  from1 = from2 = match;
+  while (seg1 || seg2)
+    {
+      if (seg1)
+	{
+	  if (seg1->type == AS_SEQUENCE)
+	    {
+	      asset = aspath_aggregate_as_set_add (aspath, asset, seg1->as[from1]);
+	      from1++;
+	      if (from1 >= seg1->length)
+		{
+		  from1 = 0;
+		  seg1 = seg1->next;
+		}
+	    }
+	  else
+	    {
+	      for (i = from1; i < seg1->length; i++)
+		asset = aspath_aggregate_as_set_add (aspath, asset, seg1->as[i]);
+
+	      from1 = 0;
+	      seg1 = seg1->next;
+	    }
+	  }
+
+      if (seg2)
+	{
+	  if (seg2->type == AS_SEQUENCE)
+	    {
+	      asset = aspath_aggregate_as_set_add (aspath, asset, seg2->as[from2]);
+	      from2++;
+	      if (from2 >= seg2->length)
+		{
+		  from2 = 0;
+		  seg2 = seg2->next;
+		}
+	    }
+	  else
+	    {
+	      for (i = from2; i < seg2->length; i++)
+		asset = aspath_aggregate_as_set_add (aspath, asset, seg2->as[i]);
+
+	      from2 = 0;
+	      seg2 = seg2->next;
+	    }
+	}
+
+      if (asset->length == 1)
+	asset->type = AS_SEQUENCE;
+      asset = NULL;
+    }
+
+  assegment_normalise (aspath->segments);
+  aspath_str_update (aspath);
+  return aspath;
+}
+
 /* When a BGP router receives an UPDATE with an MP_REACH_NLRI
    attribute, check the leftmost AS number in the AS_PATH attribute is
    or not the peer's AS number. */ 
@@ -1150,8 +1282,7 @@ aspath_private_as_check (struct aspath *aspath)
       
       for (i = 0; i < seg->length; i++)
 	{
-	  if ( (seg->as[i] < BGP_PRIVATE_AS_MIN)
-	      || (seg->as[i] > BGP_PRIVATE_AS_MAX) )
+	  if (!BGP_AS_IS_PRIVATE(seg->as[i]))
 	    return 0;
 	}
       seg = seg->next;
@@ -1248,7 +1379,18 @@ aspath_prepend (struct aspath *as1, struct aspath *as2)
   /* Delete any AS_CONFED_SEQUENCE segment from as2. */
   if (seg1->type == AS_SEQUENCE && seg2->type == AS_CONFED_SEQUENCE)
     as2 = aspath_delete_confed_seq (as2);
-
+  
+  /* as2 may have been updated */
+  seg2 = as2->segments;
+  
+  /* as2 may be empty now due to aspath_delete_confed_seq, recheck */
+  if (seg2 == NULL)
+    {
+      as2->segments = assegment_dup_all (as1->segments);
+      aspath_str_update (as2);
+      return as2;
+    }
+  
   /* Compare last segment type of as1 and first segment type of as2. */
   if (seg1->type != seg2->type)
     return aspath_merge (as1, as2);
@@ -1378,7 +1520,7 @@ static struct aspath *
 aspath_add_asns (struct aspath *aspath, as_t asno, u_char type, unsigned num)
 {
   struct assegment *assegment = aspath->segments;
-  int i;
+  unsigned i;
 
   if (assegment && assegment->type == type)
     {
@@ -1435,6 +1577,10 @@ aspath_cmp_left (const struct aspath *aspath1, const struct aspath *aspath2)
   seg1 = aspath1->segments;
   seg2 = aspath2->segments;
 
+  /* If both paths are originated in this AS then we do want to compare MED */
+  if (!seg1 && !seg2)
+    return 1;
+
   /* find first non-confed segments for each */
   while (seg1 && ((seg1->type == AS_CONFED_SEQUENCE)
 		  || (seg1->type == AS_CONFED_SET)))
@@ -1879,6 +2025,7 @@ aspath_init (void)
 void
 aspath_finish (void)
 {
+  hash_clean (ashash, (void (*)(void *))aspath_free);
   hash_free (ashash);
   ashash = NULL;
   
@@ -1913,7 +2060,7 @@ aspath_show_all_iterator (struct hash_backet *backet, struct vty *vty)
 
   as = (struct aspath *) backet->data;
 
-  vty_out (vty, "[%p:%u] (%ld) ", backet, backet->key, as->refcnt);
+  vty_out (vty, "[%p:%u] (%ld) ", (void *)backet, backet->key, as->refcnt);
   vty_out (vty, "%s%s", as->str, VTY_NEWLINE);
 }
 
