@@ -507,7 +507,7 @@ bgp_stop (struct peer *peer)
       /* Reset peer synctime */
       peer->synctime = 0;
     }
-
+  
   /* Stop read and write threads when exists. */
   BGP_READ_OFF (peer->t_read);
   BGP_WRITE_OFF (peer->t_write);
@@ -587,19 +587,41 @@ bgp_stop (struct peer *peer)
   return 0;
 }
 
+/* first-val * 2**x back-off, where x is the number of sucessive calls 
+ * originally used for peer v_start back-off 
+ */
+__attribute__((unused))
+static int
+back_off_exp2 (const int first, int val, const int max) 
+{
+  val <<= 1;
+  return (val < max ? val : max);
+}
+
+/* exponential back off, but biased downward by the initial value. 
+ * this bias is significant at lower values, and tends to
+ * insignificance fairly quickly, so it is equal to the previous at
+ * scale.  Is below first-val * 1.7**x at x == 6, and below first-val
+ * * 1.75**x at x=10.
+ *
+ * I.e., this function is useful to get slower growth for the initial
+ * points of x.
+ */
+__attribute__((unused))
+static int
+back_off_exp2_bias (const int first, int val, const int max)
+{
+  val = (val << 1) - (val > first ? first : 0);
+  return (val < max ? val : max);
+}
+
 /* BGP peer is stoped by the error. */
 static int
 bgp_stop_with_error (struct peer *peer)
 {
-  /* Double start timer. */
-  peer->v_start *= 2;
-
-  /* Overflow check. */
-  if (peer->v_start >= (60 * 2))
-    peer->v_start = (60 * 2);
-
+  peer->v_start
+   = back_off_exp2_bias (BGP_INIT_START_TIMER, peer->v_start, 60);
   bgp_stop (peer);
-
   return 0;
 }
 
@@ -634,6 +656,8 @@ bgp_stop_with_notify (struct peer *peer, u_char code, u_char sub_code)
 static int
 bgp_connect_success (struct peer *peer)
 {
+  struct peer *realpeer;
+  
   if (peer->fd < 0)
     {
       zlog_err ("bgp_connect_success peer's fd is negative value %d",
@@ -655,9 +679,29 @@ bgp_connect_success (struct peer *peer)
       else
 	zlog_debug ("%s passive open", peer->host);
     }
-
-  if (! CHECK_FLAG (peer->sflags, PEER_STATUS_ACCEPT_PEER))
-    bgp_open_send (peer);
+  
+  /* Generally we want to send OPEN ASAP. Except, some partial BGP
+   * implementations out there (e.g., conformance test tools / BGP
+   * traffic generators) seem to be a bit funny about connection collisions,
+   * and OPENs before they have sent.
+   *
+   * As a hack, delay sending OPEN on an inbound accept-peer session
+   * _IF_ we locally have an outbound connection in progress, i.e. 
+   * we're in middle of a connection collision. If we delay, we delay until
+   * an Open is received - as per old Quagga behaviour.
+   */
+  if (CHECK_FLAG (peer->sflags, PEER_STATUS_ACCEPT_PEER))
+    {
+      realpeer = peer_lookup (peer->bgp, &peer->su);
+      
+      if (realpeer->status > Idle && realpeer->status <= Established)
+        {
+          SET_FLAG (peer->sflags, PEER_STATUS_OPEN_DEFERRED);
+          return 0;
+        }
+   }
+  
+  bgp_open_send (peer);
 
   return 0;
 }
@@ -716,11 +760,11 @@ bgp_start (struct peer *peer)
     }
 
   /* Register to be notified on peer up */
-  if ((peer->ttl == 1) || (peer->gtsm_hops == 1))
+  if ((peer_ttl(peer) == 1 || peer->gtsm_hops == 1) &&
+      ! CHECK_FLAG (peer->flags, PEER_FLAG_DISABLE_CONNECTED_CHECK))
     connected = 1;
 
-  bgp_find_or_add_nexthop(family2afi(peer->su.sa.sa_family), NULL, peer,
-			  connected);
+  bgp_ensure_nexthop (NULL, peer, connected);
   status = bgp_connect (peer);
 
   switch (status)
@@ -971,7 +1015,7 @@ static const struct {
     {bgp_ignore, Idle},		/* Receive_UPDATE_message       */
     {bgp_ignore, Idle},		/* Receive_NOTIFICATION_message */
     {bgp_ignore, Idle},         /* Clearing_Completed           */
-    {bgp_start,  Connect},      /* NHT_Update                   */
+    {bgp_ignore, Idle},         /* BGP_Stop_with_error          */
   },
   {
     /* Connect */
@@ -989,7 +1033,7 @@ static const struct {
     {bgp_ignore,  Idle},	/* Receive_UPDATE_message       */
     {bgp_stop,    Idle},	/* Receive_NOTIFICATION_message */
     {bgp_ignore,  Idle},         /* Clearing_Completed           */
-    {bgp_reconnect,    Connect},/* NHT_Update                   */
+    {bgp_stop_with_error, Idle},/* BGP_Stop_with_error          */
   },
   {
     /* Active, */
@@ -1007,7 +1051,7 @@ static const struct {
     {bgp_ignore,  Idle},	/* Receive_UPDATE_message       */
     {bgp_stop_with_error, Idle}, /* Receive_NOTIFICATION_message */
     {bgp_ignore, Idle},         /* Clearing_Completed           */
-    {bgp_start,   Connect},     /* NHT_Update                   */
+    {bgp_stop_with_error, Idle},/* BGP_Stop_with_error          */
   },
   {
     /* OpenSent, */
@@ -1025,7 +1069,7 @@ static const struct {
     {bgp_fsm_event_error, Idle}, /* Receive_UPDATE_message       */
     {bgp_stop_with_error, Idle}, /* Receive_NOTIFICATION_message */
     {bgp_ignore, Idle},         /* Clearing_Completed           */
-    {bgp_ignore, OpenSent},     /* NHT_Update                   */
+    {bgp_stop_with_error, Idle},/* BGP_Stop_with_error          */
   },
   {
     /* OpenConfirm, */
@@ -1043,7 +1087,7 @@ static const struct {
     {bgp_ignore,  Idle},	/* Receive_UPDATE_message       */
     {bgp_stop_with_error, Idle}, /* Receive_NOTIFICATION_message */
     {bgp_ignore, Idle},         /* Clearing_Completed           */
-    {bgp_ignore, OpenConfirm},  /* NHT_Update                   */
+    {bgp_stop_with_error, Idle},/* BGP_Stop_with_error          */
   },
   {
     /* Established, */
@@ -1061,7 +1105,7 @@ static const struct {
     {bgp_fsm_update,           Established}, /* Receive_UPDATE_message       */
     {bgp_stop_with_error,         Clearing}, /* Receive_NOTIFICATION_message */
     {bgp_ignore,                      Idle}, /* Clearing_Completed           */
-    {bgp_ignore,               Established}, /* NHT_Update                   */
+    {bgp_stop_with_error,         Clearing}, /* BGP_Stop_with_error          */
   },
   {
     /* Clearing, */
@@ -1079,7 +1123,7 @@ static const struct {
     {bgp_stop,			Clearing},	/* Receive_UPDATE_message       */
     {bgp_stop,			Clearing},	/* Receive_NOTIFICATION_message */
     {bgp_clearing_completed,    Idle},		/* Clearing_Completed           */
-    {bgp_ignore,                Clearing},      /* NHT_Update                   */
+    {bgp_stop_with_error,       Clearing},      /* BGP_Stop_with_error          */
   },
   {
     /* Deleted, */
@@ -1097,7 +1141,7 @@ static const struct {
     {bgp_ignore,  Deleted},	/* Receive_UPDATE_message       */
     {bgp_ignore,  Deleted},	/* Receive_NOTIFICATION_message */
     {bgp_ignore,  Deleted},	/* Clearing_Completed           */
-    {bgp_ignore,  Deleted},     /* NHT_Update                   */
+    {bgp_ignore,  Deleted},     /* BGP_Stop_with_error          */
   },
 };
 
@@ -1118,7 +1162,7 @@ static const char *bgp_event_str[] =
   "Receive_UPDATE_message",
   "Receive_NOTIFICATION_message",
   "Clearing_Completed",
-  "NHT_Update",
+  "BGP_Stop_with_error",
 };
 
 /* Execute event process. */
