@@ -1625,6 +1625,16 @@ DEFUN (link_params,
   return CMD_SUCCESS;
 }
 
+DEFUN (exit_link_params,
+       exit_link_params_cmd,
+       "exit-link-params",
+       "Exit from Link Params configuration mode\n")
+{
+  if (vty->node == LINK_PARAMS_NODE)
+    vty->node = INTERFACE_NODE;
+  return CMD_SUCCESS;
+}
+
 /* Specific Traffic Engineering parameters commands */
 DEFUN (link_params_enable,
        link_params_enable_cmd,
@@ -2636,6 +2646,7 @@ link_params_config_write (struct vty *vty, struct interface *ifp)
   if (IS_PARAM_SET(iflp, LP_RMT_AS))
     vty_out(vty, "  neighbor %s as %u%s", inet_ntoa(iflp->rmt_ip),
         iflp->rmt_as, VTY_NEWLINE);
+  vty_out(vty, "  exit-link-params%s", VTY_NEWLINE);
   return 0;
 }
 
@@ -2786,13 +2797,21 @@ zebra_if_init (void)
   install_element(LINK_PARAMS_NODE, &link_params_max_rsv_bw_cmd);
   install_element(LINK_PARAMS_NODE, &link_params_unrsv_bw_cmd);
   install_element(LINK_PARAMS_NODE, &link_params_admin_grp_cmd);
+  install_element(LINK_PARAMS_NODE, &no_link_params_admin_grp_cmd);
   install_element(LINK_PARAMS_NODE, &link_params_inter_as_cmd);
   install_element(LINK_PARAMS_NODE, &no_link_params_inter_as_cmd);
   install_element(LINK_PARAMS_NODE, &link_params_delay_cmd);
+  install_element(LINK_PARAMS_NODE, &no_link_params_delay_cmd);
   install_element(LINK_PARAMS_NODE, &link_params_delay_mm_cmd);
   install_element(LINK_PARAMS_NODE, &link_params_delay_var_cmd);
+  install_element(LINK_PARAMS_NODE, &no_link_params_delay_var_cmd);
   install_element(LINK_PARAMS_NODE, &link_params_pkt_loss_cmd);
+  install_element(LINK_PARAMS_NODE, &no_link_params_pkt_loss_cmd);
   install_element(LINK_PARAMS_NODE, &link_params_ava_bw_cmd);
+  install_element(LINK_PARAMS_NODE, &no_link_params_ava_bw_cmd);
   install_element(LINK_PARAMS_NODE, &link_params_res_bw_cmd);
+  install_element(LINK_PARAMS_NODE, &no_link_params_res_bw_cmd);
   install_element(LINK_PARAMS_NODE, &link_params_use_bw_cmd);
+  install_element(LINK_PARAMS_NODE, &no_link_params_use_bw_cmd);
+  install_element(LINK_PARAMS_NODE, &exit_link_params_cmd);
 }
